----------------------------------------------------------------------------------
-- Company: 
-- Engineer: 
-- 
-- Create Date:    14:07:47 10/07/2013 
-- Design Name: 
-- Module Name:    TCPIP_if - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;
use IEEE.std_logic_misc.all;
use IEEE.numeric_std.all;
use work.amc13_pack.all;

-- Uncomment the following library declaration if using
-- arithmetic functions with Signed or Unsigned values
--use IEEE.NUMERIC_STD.ALL;

-- Uncomment the following library declaration if instantiating
-- any Xilinx primitives in this code.
library UNISIM;
use UNISIM.VComponents.all;
Library UNIMACRO;
use UNIMACRO.vcomponents.all;

entity DAQLSC_if is
    Port ( sysclk : in  STD_LOGIC;
           clk125 : in  STD_LOGIC;
           DRPclk : in  STD_LOGIC;
           reset : in  STD_LOGIC;
           gtx_reset : in  STD_LOGIC;
           rstCntr : in  STD_LOGIC;
           test : in  STD_LOGIC;
					 enSFP : IN std_logic_vector(3 downto 0);
					 LSC_ID : IN std_logic_vector(15 downto 0);
					 SFP_down : OUT std_logic_vector(2 downto 0);
           inc_ddr_pa : in  STD_LOGIC;
-- event data in
					 evt_data_rdy : in std_logic_vector(2 downto 0);
					 EventData_in : in array3X66;
					 EventData_we : in   std_logic_VECTOR(2 downto 0);
					 EventData_re : out   std_logic_VECTOR(2 downto 0); -- 
					 evt_buf_full : out std_logic_vector(2 downto 0);
					 buf_rqst : in std_logic_vector(1 downto 0);
           MonBuf_avl : out  STD_LOGIC;
					 WrtMonBufDone : out  STD_LOGIC;
           TCPBuf_avl : out  STD_LOGIC;
-- ddr wportA status
					 wport_rdy : in std_logic_vector(2 downto 0);
					 wport_FIFO_full : in std_logic_vector(2 downto 0);
-- ddr wportB signals in sysclk domain
					 TCP_dout : out std_logic_vector(31 downto 0); -- TCP data are written in unit of 32-bit words
					 TCP_channel : out std_logic_vector(1 downto 0); -- Each entry has four 32bit words, each address saves two entries. Addresses are kept in ddr_wportB
					 TCP_wrqst : out STD_LOGIC;
					 TCP_re	: in STD_LOGIC;
-- ddr rport signals
					 TCP_raddr : out  STD_LOGIC_VECTOR(23 downto 0);
					 TCP_length	: out  STD_LOGIC_VECTOR(11 downto 0); -- in 32 bit word
					 TCP_rrqst : out  STD_LOGIC;
					 TCP_rack	: in  STD_LOGIC;
					 TCP_din	: in  STD_LOGIC_VECTOR(31 downto 0); -- TCP data are written in unit of 32-bit words
					 TCP_din_valid	: in  STD_LOGIC;
					 TCP_lastword	: in  STD_LOGIC;
-- signal to ddr_if, AMC_if to start moving data
					 EventBufAddr_we : out std_logic_VECTOR(2 downto 0);
					 EventBufAddr : out array3X17;
-- SFP ports
           SFP0_RXN : in  STD_LOGIC;
           SFP0_RXP : in  STD_LOGIC;
           SFP1_RXN : in  STD_LOGIC;
           SFP1_RXP : in  STD_LOGIC;
           SFP2_RXN : in  STD_LOGIC;
           SFP2_RXP : in  STD_LOGIC;
           SFP0_TXN : out  STD_LOGIC;
           SFP0_TXP : out  STD_LOGIC;
           SFP1_TXN : out  STD_LOGIC;
           SFP1_TXP : out  STD_LOGIC;
           SFP2_TXN : out  STD_LOGIC;
           SFP2_TXP : out  STD_LOGIC;
           SFP_REFCLK : in  STD_LOGIC;
--	ipbus signals
					 ipb_clk : in  STD_LOGIC;
					 ipb_write : in  STD_LOGIC;
					 ipb_strobe : in  STD_LOGIC;
					 ipb_addr	: in  STD_LOGIC_VECTOR(31 downto 0);
					 ipb_wdata : in  STD_LOGIC_VECTOR(31 downto 0);
					 ipb_rdata : out  STD_LOGIC_VECTOR(31 downto 0);
					 ipb_ack : out  STD_LOGIC
					 );
end DAQLSC_if;

architecture Behavioral of DAQLSC_if is
COMPONENT DaqLSC
	PORT(
		sys_reset : IN std_logic;
		sys_clk : IN std_logic;
		DRP_clk : IN std_logic;
		clk125 : IN std_logic;
		LinkWe : IN std_logic;
		LinkCtrl : IN std_logic;
		LinkData : IN std_logic_vector(63 downto 0);
		srcID : IN std_logic_vector(15 downto 0);
		status_ce : IN std_logic;
		status_addr : IN std_logic_vector(15 downto 0);
		gtx_reset : IN std_logic;
		gtx_refclk : IN std_logic;
		sfp_rxn : IN std_logic;
		sfp_rxp : IN std_logic;          
		LinkDown : OUT std_logic;
		LinkFull : OUT std_logic;
		ack_cnt : OUT std_logic;
		pckt_cnt : OUT std_logic;
		retransmit_cnt : OUT std_logic;
		event_cnt : OUT std_logic;
		sync_loss : OUT std_logic;
		status_port : OUT std_logic_vector(63 downto 0);
		txusrclk_o : OUT std_logic;
		rxusrclk_o : OUT std_logic;
		sfp_txn : OUT std_logic;
		sfp_txp : OUT std_logic
		);
END COMPONENT;
signal resetSyncRegs : std_logic_vector(2 downto 0) := (others => '0');
signal inc_ddr_paSyncRegs : std_logic_vector(3 downto 0) := (others => '0');
signal bldr2SFP_sel : array3X2 := (others => (others => '0'));
signal SFP2bldr_sel : array3X2 := (others => (others => '0'));
signal EventBufAddr_we_i : std_logic_vector(2 downto 0) := (others => '0');
signal ReadBusy : std_logic_vector(2 downto 0) := (others => '0');
signal sysDIV2 : std_logic := '0';
signal TCPBufCnt : std_logic_vector(12 downto 0) := (others => '0');
signal MonBufCnt : std_logic_vector(12 downto 0) := (others => '0');
signal WrtMonBuf : std_logic_vector(2 downto 0) := (others => '0');
signal NXT_MonBuf : array4X12 := (others => (others => '0'));
signal NXT_MonBufMatch : std_logic_vector(2 downto 0) := (others => '0');
signal AddrOffset : array3X10 := (others => (others => '0'));
signal header : std_logic_vector(2 downto 0) := (others => '0');
signal header_q : std_logic_vector(2 downto 0) := (others => '0');
signal sfp_rxn : std_logic_vector(2 downto 0) := (others => '0');
signal sfp_rxp : std_logic_vector(2 downto 0) := (others => '0');
signal sfp_txn : std_logic_vector(2 downto 0) := (others => '0');
signal sfp_txp : std_logic_vector(2 downto 0) := (others => '0');
signal sync_loss : std_logic_vector(2 downto 0) := (others => '0');
signal event_cnt : std_logic_vector(2 downto 0) := (others => '0');
signal retransmit_cnt : std_logic_vector(2 downto 0) := (others => '0');
signal pckt_cnt : std_logic_vector(2 downto 0) := (others => '0');
signal ack_cnt : std_logic_vector(2 downto 0) := (others => '0');
signal LinkFull_n : std_logic_vector(2 downto 0) := (others => '0');
--signal srcID : std_logic_vector(15 downto 0) := (others => '0');
signal LinkCtrl : std_logic_vector(2 downto 0) := (others => '0');
signal LinkWe : std_logic_vector(2 downto 0) := (others => '0');
signal DaqLSC_status : array3X64 := (others => (others => '0'));
signal LinkData : array3X65 := (others => (others => '0'));
signal ack_cntr : array3X32 := (others => (others => '0'));
signal pckt_cntr : array3X32 := (others => (others => '0'));
signal retransmit_cntr : array3X32 := (others => (others => '0'));
signal sync_loss_cntr : array3X32 := (others => (others => '0'));
signal LSC_event_cntr : array3X32 := (others => (others => '0'));
signal bldr_evt_cntr : array3X32 := (others => (others => '0'));
signal bldr_word_cntr : array3X32 := (others => (others => '0'));
signal event_cntr : array3X32 := (others => (others => '0'));
signal word_cntr : array3X32 := (others => (others => '0'));
signal EventBufAddr_we_cntr : array3X32 := (others => (others => '0'));
signal WrtMonBufDone_i : std_logic := '0';
signal WrtMonBufDone_cntr : std_logic_vector(31 downto 0) := (others => '0');
begin
TCPBuf_avl <= '1';
TCP_rrqst <= '0';
TCP_wrqst <= '0';
TCP_dout <= (others => '0');
TCP_channel <= (others => '0');
TCP_raddr <= (others => '0');
TCP_length <= (others => '0');
ipb_ack <= '0';
WrtMonBufDone <= WrtMonBufDone_i;
g_DAQLSC : for i in 0 to 2 generate
	i_DaqLSC: DaqLSC PORT MAP(
		sys_reset => resetSyncRegs(2),
		sys_clk => sysclk,
		DRP_clk => DRPclk,
		clk125 => clk125,
		LinkWe => LinkWe(i),
		LinkCtrl => LinkCtrl(i),
		LinkData => LinkData(i)(63 downto 0),
		srcID => LSC_ID(15 downto 2) & CONV_STD_LOGIC_VECTOR(i, 2),
		LinkDown => SFP_down(i),
		LinkFull => LinkFull_n(i),
		ack_cnt => ack_cnt(i),
		pckt_cnt => pckt_cnt(i),
		retransmit_cnt => retransmit_cnt(i),
		event_cnt => event_cnt(i),
		sync_loss => sync_loss(i),
		status_ce => '0',
		status_addr => (others => '0'),
		status_port => DaqLSC_status(i),
		txusrclk_o => open,
		rxusrclk_o => open,
		gtx_reset => gtx_reset,
		gtx_refclk => SFP_REFCLK,
		sfp_rxn => sfp_rxn(i),
		sfp_rxp => sfp_rxp(i),
		sfp_txn => sfp_txn(i),
		sfp_txp => sfp_txp(i)
	);
end generate;
sfp_rxn(0) <= SFP0_RXN;
sfp_rxp(0) <= SFP0_RXP;
SFP0_TXN <= sfp_txn(0);
SFP0_TXP <= sfp_txp(0);
sfp_rxn(1) <= SFP1_RXN;
sfp_rxp(1) <= SFP1_RXP;
SFP1_TXN <= sfp_txn(1);
SFP1_TXP <= sfp_txp(1);
sfp_rxn(2) <= SFP2_RXN;
sfp_rxp(2) <= SFP2_RXP;
SFP2_TXN <= sfp_txn(2);
SFP2_TXP <= sfp_txp(2);
MonBuf_avl <= not MonBufCnt(12);
EventBufAddr_we <= EventBufAddr_we_i;
process(sysclk,reset)
begin
	if(reset = '1')then
		resetSyncRegs <= (others => '1');
	elsif(sysclk'event and sysclk = '1')then
		resetSyncRegs <= resetSyncRegs(1 downto 0) & '0';
	end if;
end process;
process(sysclk)
begin
	if(sysclk'event and sysclk = '1')then
		if(resetSyncRegs(2) = '1')then
			ReadBusy <= (others => '0');
			EventData_re <= (others => '0');
			EventBufAddr_we_i <= (others => '0');
			MonBufCnt <= (others => '0');
			NXT_MonBufMatch <= (others => '1');
			WrtMonBuf <= (others => '0');
			WrtMonBufDone_i <= '0';
			NXT_MonBuf <= (others => (others => '0'));
			inc_ddr_paSyncRegs <= (others => '0');
		else
			for i in 0 to 2 loop
				if(EVENTdata_in(i)(64) = '1' and EVENTdata_we(i) = '1')then
					ReadBusy(i) <= '0';
				elsif(WrtMonBuf(i) = '0' and evt_data_rdy(i) = '1' and wport_rdy(i) = '1')then
					ReadBusy(i) <= '1';
				end if;
				EventData_re(i) <= evt_data_rdy(i) and wport_rdy(i) and not ReadBusy(i);
				if(EventData_in(i)(65) = '0')then
					EventBufAddr_we_i(i) <= not WrtMonBuf(i) and evt_data_rdy(i) and wport_rdy(i) and not ReadBusy(i);
				else
					EventBufAddr_we_i(i) <= '0';
				end if;
			end loop;
			if(buf_rqst(0) = '1' and inc_ddr_paSyncRegs(3 downto 2) /= "10")then
				MonBufCnt <= MonBufCnt + 1;
			elsif(buf_rqst(0) = '0' and inc_ddr_paSyncRegs(3 downto 2) = "10")then
				MonBufCnt <= MonBufCnt - 1;
			end if;
			if(NXT_MonBuf(0) = NXT_MonBuf(3))then
				NXT_MonBufMatch(0) <= '1';
			else
				NXT_MonBufMatch(0) <= '0';
			end if;
			if((EnSFP(2 downto 0) = "111" or EnSFP(2 downto 0) = "011" or EnSFP(2 downto 0) = "110" or EnSFP(2 downto 0) = "101") and NXT_MonBuf(1) = NXT_MonBuf(3))then
				NXT_MonBufMatch(1) <= '1';
			else
				NXT_MonBufMatch(1) <= '0';
			end if;
			if(EnSFP(2 downto 0) = "111" and NXT_MonBuf(2) = NXT_MonBuf(3))then
				NXT_MonBufMatch(2) <= '1';
			else
				NXT_MonBufMatch(2) <= '0';
			end if;
			if(NXT_MonBufMatch = "000" and sysDIV2 = '0')then
				NXT_MonBuf(3) <= NXT_MonBuf(3) + 1;
				WrtMonBufDone_i <= '1';
			else
				WrtMonBufDone_i <= '0';
			end if;
			for i in 0 to 2 loop
				if(EventBufAddr_we_i(i) = '1' and EventData_in(i)(65) = '0')then
					WrtMonBuf(i) <= '1';
				elsif(wport_rdy(i) = '1')then
					WrtMonBuf(i) <= '0';
				end if;
				if(WrtMonBuf(i) = '1' and wport_rdy(i) = '1')then
					NXT_MonBuf(i) <= NXT_MonBuf(i) + 1;
				end if;
			end loop;
			inc_ddr_paSyncRegs <= inc_ddr_paSyncRegs(2 downto 0) & inc_ddr_pa;
		end if;
		case EnSFP(2 downto 0) is
			when "011" | "101" | "110" => AddrOffset(0)(9 downto 6) <= x"8";
			when "111" => AddrOffset(0)(9 downto 6) <= x"5";
			when others => AddrOffset(0)(9 downto 6) <= x"0";
		end case;
		AddrOffset(1)(9 downto 6) <= x"0";
		AddrOffset(2)(9 downto 6) <= x"a";
		for i in 0 to 2 loop
			EventBufAddr(i) <= '0' & NXT_MonBuf(i) & AddrOffset(i)(9 downto 6);
		end loop;
	end if;
end process;
process(sysclk)
begin
	if(sysclk'event and sysclk = '1')then
		sysDIV2 <= not sysDIV2;
		for i in 0 to 2 loop
			if(resetSyncRegs(2) = '1')then
				header(i) <= '1';
				header_q(i) <= '0';
			elsif(EventData_we(i) = '1')then
				header(i) <= EventData_in(i)(64) or (header(i) and not header_q(i));
				header_q(i) <= header(i);
			end if;
		end loop;
--		case EnSFP(2 downto 0) is
--			when "011" | "101" | "111" => bldr2SFP_sel(0) <= "01";
--			when others => bldr2SFP_sel(0) <= "00";
--		end case;
--		case EnSFP(2 downto 0) is
--			when "010" | "011" | "111" => bldr2SFP_sel(1) <= "00";
--			when others => bldr2SFP_sel(1) <= "01";
--		end case;
--		case EnSFP(2 downto 0) is
--			when "111" => bldr2SFP_sel(2) <= "10";
--			when others => bldr2SFP_sel(2) <= "00";
--		end case;
		if(EnSFP(2 downto 0) = "001")then
			LinkData(0) <= EventData_in(0)(64 downto 0);
			LinkCtrl(0) <= not(header(0) or EventData_in(0)(64));
			LinkWe(0) <= EventData_we(0) and not EnSFP(3);
		else
			LinkData(0) <= EventData_in(1)(64 downto 0);
			LinkCtrl(0) <= not(header(1) or EventData_in(1)(64));
			LinkWe(0) <= EventData_we(1) and EnSFP(0) and not EnSFP(3);
		end if;
		if(EnSFP(2 downto 0) = "110")then
			LinkData(1) <= EventData_in(1)(64 downto 0);
			LinkCtrl(1) <= not(header(1) or EventData_in(1)(64));
			LinkWe(1) <= EventData_we(1) and not EnSFP(3);
		else
			LinkData(1) <= EventData_in(0)(64 downto 0);
			LinkCtrl(1) <= not(header(0) or EventData_in(0)(64));
			LinkWe(1) <= EventData_we(0) and EnSFP(1) and not EnSFP(3);
		end if;
		if(EnSFP(2 downto 0) = "111")then
			LinkData(2) <= EventData_in(2)(64 downto 0);
			LinkCtrl(2) <= not(header(2) or EventData_in(2)(64));
			LinkWe(2) <= EventData_we(2) and not EnSFP(3);
		else
			LinkData(2) <= EventData_in(0)(64 downto 0);
			LinkCtrl(2) <= not(header(0) or EventData_in(0)(64));
			LinkWe(2) <= EventData_we(0) and EnSFP(2) and not EnSFP(3);
		end if;
--		for i in 0 to 1 loop
--			LinkData(i) <= EventData_in(conv_integer(bldr2SFP_sel(i)))(63 downto 0);
--			LinkCtrl(i) <= not(header(conv_integer(bldr2SFP_sel(i))) or EventData_in(i)(64));
--			LinkWe(i) <= EventData_we(conv_integer(bldr2SFP_sel(i)));
--		end loop;
--		if(bldr2SFP_sel(2)(1) = '0')then
--			LinkData(2) <= EventData_in(0)(63 downto 0);
--			LinkCtrl(2) <= not(header(0) or EventData_in(0)(64));
--			LinkWe(2) <= EventData_we(0);
--		else
--			LinkData(2) <= EventData_in(2)(63 downto 0);
--			LinkCtrl(2) <= not(header(2) or EventData_in(2)(64));
--			LinkWe(2) <= EventData_we(2);
--		end if;
		case EnSFP(2 downto 0) is
			when "010" | "011" | "111" => SFP2bldr_sel(0) <= "01";
			when "100" | "101" | "110" => SFP2bldr_sel(0) <= "10";
			when others => SFP2bldr_sel(0) <= "00";
		end case;
		case EnSFP(2 downto 0) is
			when "010" | "011" | "101" | "111" => SFP2bldr_sel(1) <= "00";
			when others => SFP2bldr_sel(1) <= "01";
		end case;
		case EnSFP(2 downto 0) is
			when "100" | "110" => SFP2bldr_sel(2) <= "00";
			when "101" => SFP2bldr_sel(2) <= "01";
			when others => SFP2bldr_sel(2) <= "10";
		end case;
		evt_buf_full(0) <= (not LinkFull_n(conv_integer(SFP2bldr_sel(0))) and EnSFP(conv_integer(SFP2bldr_sel(0))) and not EnSFP(3)) or wport_FIFO_full(0);
		if(SFP2bldr_sel(1)(0) = '0')then
			evt_buf_full(1) <= (not LinkFull_n(0) and EnSFP(0) and not EnSFP(3)) or wport_FIFO_full(1);
		else
			evt_buf_full(1) <= (not LinkFull_n(1) and EnSFP(1) and not EnSFP(3)) or wport_FIFO_full(1);
		end if;
		evt_buf_full(2) <= (not LinkFull_n(conv_integer(SFP2bldr_sel(2))) and EnSFP(conv_integer(SFP2bldr_sel(2))) and not EnSFP(3))  or wport_FIFO_full(2);
	end if;
end process;
process(clk125,rstCntr)
begin
	if(rstCntr = '1')then
		ack_cntr <= (others => (others => '0'));
		pckt_cntr <= (others => (others => '0'));
		retransmit_cntr <= (others => (others => '0'));
	elsif(clk125'event and clk125 = '1')then
		for i in 0 to 2 loop
			if(ack_cnt(i) = '1')then
				ack_cntr(i) <= ack_cntr(i) + 1;
			end if;
			if(pckt_cnt(i) = '1')then
				pckt_cntr(i) <= pckt_cntr(i) + 1;
			end if;
			if(retransmit_cnt(i) = '1')then
				retransmit_cntr(i) <= retransmit_cntr(i) + 1;
			end if;
		end loop;
	end if;
end process;
process(clk125,rstCntr)
begin
	if(rstCntr = '1')then
		sync_loss_cntr <= (others => (others => '0'));
	elsif(clk125'event and clk125 = '1')then
		for i in 0 to 2 loop
			if(sync_loss(i) = '1')then
				sync_loss_cntr(i) <= sync_loss_cntr(i) + 1;
			end if;
		end loop;
	end if;
end process;
process(sysclk,rstCntr)
begin
	if(rstCntr = '1')then
		LSC_event_cntr <= (others => (others => '0'));
		bldr_evt_cntr <= (others => (others => '0'));
		bldr_word_cntr <= (others => (others => '0'));
		event_cntr <= (others => (others => '0'));
		word_cntr <= (others => (others => '0'));
		EventBufAddr_we_cntr <= (others => (others => '0'));
		WrtMonBufDone_cntr <= (others => '0');
	elsif(sysclk'event and sysclk = '1')then
		for i in 0 to 2 loop
			if(event_cnt(i) = '1')then
				LSC_event_cntr(i) <= LSC_event_cntr(i) + 1;
			end if;
			if(LinkWe(i) = '1')then
				word_cntr(i) <= word_cntr(i) + 1;
			end if;
			if(LinkWe(i) = '1' and LinkData(i)(64) = '1')then
				event_cntr(i) <= event_cntr(i) + 1;
			end if;
			if(EventData_we(i) = '1' and EventData_in(i)(64) = '1')then
				bldr_evt_cntr(i) <= bldr_evt_cntr(i) + 1;
			end if;
			if(EventData_we(i) = '1')then
				bldr_word_cntr(i) <= bldr_word_cntr(i) + 1;
			end if;
			if(EventBufAddr_we_i(i) = '1')then
				EventBufAddr_we_cntr(i) <= EventBufAddr_we_cntr(i) + 1;
			end if;
		end loop;
--		if((bldr2SFP_sel(0)(0) = '0' and EventData_we(0) = '1') or EventData_we(1) = '1')then
--			word_cntr(0) <= word_cntr(0) + 1;
--		end if;
--		if(bldr2SFP_sel(0)(0) = '1' and EventData_we(0) = '1' and EventData_in(0)(64) = '1')then
--			event_cntr(1) <= event_cntr(1) + 1;
--		end if;
--		if((bldr2SFP_sel(0)(0) = '1' and EventData_we(0) = '1'))then
--			word_cntr(1) <= word_cntr(1) + 1;
--		end if;
--		if(EventData_we(2) = '1' and EventData_in(2)(64) = '1')then
--			event_cntr(2) <= event_cntr(2) + 1;
--		end if;
--		if(EventData_we(2) = '1')then
--			word_cntr(2) <= word_cntr(2) + 1;
--		end if;
		if(WrtMonBufDone_i = '1')then
			WrtMonBufDone_cntr <= WrtMonBufDone_cntr + 1;
		end if;
	end if;
end process;
process(ipb_clk)
begin
	if(ipb_clk'event and ipb_clk = '1')then
		if(ipb_addr(15 downto 6) = LSC_addr(15 downto 6))then
			case ipb_addr(5 downto 4) is
				when "00" =>
					case ipb_addr(3 downto 0) is
						when x"0" => ipb_rdata <= ack_cntr(0);
						when x"1" => ipb_rdata <= pckt_cntr(0);
						when x"2" => ipb_rdata <= retransmit_cntr(0);
						when x"3" => ipb_rdata <= LSC_event_cntr(0);
--						when x"4" => ipb_rdata <= bldr_evt_cntr(0);
						when x"5" => ipb_rdata <= sync_loss_cntr(0);
						when x"6" => ipb_rdata <= DaqLSC_status(0)(31 downto 0);
						when x"7" => ipb_rdata <= DaqLSC_status(0)(63 downto 32);
						when x"8" => ipb_rdata <= ack_cntr(1);
						when x"9" => ipb_rdata <= pckt_cntr(1);
						when x"a" => ipb_rdata <= retransmit_cntr(1);
						when x"b" => ipb_rdata <= LSC_event_cntr(1);
--						when x"c" => ipb_rdata <= bldr_evt_cntr(1);
						when x"d" => ipb_rdata <= sync_loss_cntr(1);
						when x"e" => ipb_rdata <= DaqLSC_status(1)(31 downto 0);
						when others => ipb_rdata <= DaqLSC_status(1)(63 downto 32);
					end case;
				when "01" =>
					case ipb_addr(3 downto 0) is
						when x"0" => ipb_rdata <= ack_cntr(2);
						when x"1" => ipb_rdata <= pckt_cntr(2);
						when x"2" => ipb_rdata <= retransmit_cntr(2);
						when x"3" => ipb_rdata <= LSC_event_cntr(2);
--						when x"4" => ipb_rdata <= bldr_evt_cntr(2);
						when x"5" => ipb_rdata <= sync_loss_cntr(2);
						when x"6" => ipb_rdata <= DaqLSC_status(2)(31 downto 0);
						when x"7" => ipb_rdata <= DaqLSC_status(2)(63 downto 32);
						when others => ipb_rdata <= (others => '0');
					end case;
				when "10" =>
					case ipb_addr(3 downto 0) is
						when x"0" => ipb_rdata <= bldr_evt_cntr(0);
						when x"1" => ipb_rdata <= bldr_evt_cntr(1);
						when x"2" => ipb_rdata <= bldr_evt_cntr(2);
						when x"4" => ipb_rdata <= bldr_word_cntr(0);
						when x"5" => ipb_rdata <= bldr_word_cntr(1);
						when x"6" => ipb_rdata <= bldr_word_cntr(2);
						when others => ipb_rdata <= (others => '0');
					end case;
				when others =>
					case ipb_addr(3 downto 0) is
--						when x"0" => ipb_rdata <= "000" & MonBufCnt(1) & "000" & MonBufCnt;
						when x"1" => ipb_rdata <= x"0000" & "000" & MonBufCnt;
						when x"2" => ipb_rdata <= x"0" & NXT_MonBuf(1) & x"0" & NXT_MonBuf(0);
						when x"3" => ipb_rdata <= x"0" & NXT_MonBuf(3) & x"0" & NXT_MonBuf(2);
						when x"4" => ipb_rdata <= x"00" & '0' & header & '0' & LinkFull_n & '0' & ReadBusy & '0' & evt_data_rdy & '0' & wport_FIFO_full & '0' & wport_rdy;
						when x"5" => ipb_rdata <= event_cntr(0);
						when x"6" => ipb_rdata <= event_cntr(1);
						when x"7" => ipb_rdata <= event_cntr(2);
						when x"8" => ipb_rdata <= word_cntr(0);
						when x"9" => ipb_rdata <= word_cntr(1);
						when x"a" => ipb_rdata <= word_cntr(2);
						when x"b" => ipb_rdata <= EventBufAddr_we_cntr(0);
						when x"c" => ipb_rdata <= EventBufAddr_we_cntr(1);
						when x"d" => ipb_rdata <= EventBufAddr_we_cntr(2);
						when x"e" => ipb_rdata <= WrtMonBufDone_cntr;
						when others => ipb_rdata <= (others => '0');
					end case;
			end case;
		else
			ipb_rdata <= (others => '0');
		end if;
	end if;
end process;
end Behavioral;

